--[[--------------------------------------------------------------------
	PerfectRaid_ResInfo
	Shows casting and available resurrections in PerfectRaid.
	by Phanx <addons@phanx.net>
	See the accompanying README file for more information.
	http://www.wowinterface.com/downloads/info22277-PerfectRaidResInfo.html
	http://www.curse.com/addons/wow/perfectraid-resinfo
----------------------------------------------------------------------]]

local _, L = ...
local Addon = PerfectRaid:NewModule("PerfectRaid-ResInfo")
local LibResInfo = LibStub("LibResInfo-1.0")

local statusColor = {
	CASTING = "|cffffff33",
	PENDING = "|cff33ff33",
	SELFRES = "|cffcc33ff",
}

function Addon:Initialize()
	PerfectRaid.defaults.profile.ResInfoEnabled = true
	PerfectRaid.defaults.profile.ResInfoCastingColor = statusColor.CASTING
	PerfectRaid.defaults.profile.ResInfoPendingColor = statusColor.PENDING
	PerfectRaid.defaults.profile.ResInfoSelfResColor = statusColor.SELFRES

	self:RegisterMessage("DONGLE_PROFILE_CHANGED")
end

function Addon:Enable()
	if not PerfectRaid.db.profile.ResInfoEnabled then return end
	LibResInfo.RegisterAllCallbacks(self, "UpdateUnit")
end

function Addon:Disable()
	LibResInfo.UnregisterAllCallbacks(self)
end

function Addon:ConfigureButton(button)
	button.res = button:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
end

function Addon:DONGLE_PROFILE_CHANGED(event, addon, svname)
	if svname ~= "PerfectRaidDB" then return end
	local profile = PerfectRaid.db.profile

	if profile.HealEnabled then
		self:Enable()
	else
		self:Disable()
	end

	for unit, list in pairs(PerfectRaid.frames) do
		for button in pairs(list) do
			self:UpdateButtonLayout(button, profile.headers[parent.idx])
		end
	end
end

function Addon:UpdateButtonLayout(button, options)
	button.res:ClearAllPoints()
	if options.alignright then
		button.res:SetPoint("RIGHT", button.leftbox, "RIGHT", -2, 0)
	else
		button.res:SetPoint("LEFT", button.rightbox, "LEFT", 2, 0)
	end
end

function Addon:ShowButton(button)
	self:UpdateUnit("ShowButton", button:GetAttribute("unit"))
end

function Addon:FullUpdate()
	for unit in pairs(PerfectRaid.frames) do
		self:UpdateUnit("FullUpdate", unit)
	end
end

function Addon:UpdateUnit(event, unit)
	if strsub(event, 1, 18) == "LibResInfo_MassRes" then
		return self:FullUpdate()
	end

	if not unit or not PerfectRaid.frames[unit] then return end

	local status, endTime, casterUnit, casterGUID = LibResInfo:UnitHasIncomingRes(unit)
	for button in pairs(PerfectRaid.frames[unit]) do
		if status then
			if status == "MASSRES" then
				status = "CASTING"
			end
			button.aura:Hide()
			button.res:Show()
			button.res:SetFormattedText("%s%s|r", statusColor[status], L[status])
		else
			button.aura:Show()
			button.res:Hide()
			button.res:SetText("")
		end
	end
end

------------------------------------------------------------------------

local options
function Addon:CreateOptions(parent)
	options = CreateFrame("Frame", "PROptions_ResInfo", PROptions)
	options:SetScript("OnShow", function() self:OnShow() end)

	parent:AddOptionsTab(L.ResInfo, options)

	local enabled = CreateFrame("CheckButton", "PRResInfo_Enabled", options, "InterfaceOptionsCheckButtonTemplate")
	enabled:SetPoint("TOPLEFT")
	enabled.Text:SetFontObject(GameFontHighlightSmall)
	enabled.Text:SetText(L.Enable)
	options.Enabled = enabled

	local castingcolor = LibStub("PhanxConfig-ColorPicker").CreateColorPicker(options, L.CastingColor)
	castingcolor:SetPoint("TOPLEFT", enabled, "BOTTOMLEFT", 0, -15)
	castingcolor.label:SetFontObject(GameFontHighlightSmall)
	options.CastingColor = castingcolor

	local pendingcolor = LibStub("PhanxConfig-ColorPicker").CreateColorPicker(options, L.PendingColor)
	pendingcolor:SetPoint("TOPLEFT", castingcolor, "BOTTOMLEFT", 0, -15)
	pendingcolor.label:SetFontObject(GameFontHighlightSmall)
	options.PendingColor = pendingcolor

	local selfrescolor = LibStub("PhanxConfig-ColorPicker").CreateColorPicker(options, L.SelfResColor)
	selfrescolor:SetPoint("TOPLEFT", pendingcolor, "BOTTOMLEFT", 0, -15)
	selfrescolor.label:SetFontObject(GameFontHighlightSmall)
	selfrescolor:Hide() -- NYI in LibResInfo
	options.SelfResColor = selfrescolor

	local cancel = CreateFrame("Button", "PRResInfo_Cancel", options, "PRButtonTemplate")
	cancel:SetText(PerfectRaidLocals.Cancel)
	cancel:SetPoint("BOTTOMRIGHT", 0, 5)
	cancel:SetScript("OnClick", function() self:OnShow() end)

	local save = CreateFrame("Button", "PRResInfo_Save", options, "PRButtonTemplate")
	save:SetText(PerfectRaidLocals.Save)
	save:SetPoint("BOTTOMRIGHT", cancel, "BOTTOMLEFT", -10, 0)
	save:SetScript("OnClick", function() self:SaveOptions() end)
end

local function HexToRGB(hex)
	local r, g, b = strmatch(hex, "(%x%x)(%x%x)(%x%x)$")
	return tonumber(r, 16) / 255, tonumber(g, 16) / 255, tonumber(b, 16) / 255
end

local function RGBToHex(r, g, b)
	return format("|cff%02x%02x%02x", r * 255, g * 255, b * 255)
end

function Addon:OnShow()
	local profile = PerfectRaid.db.profile

	options.Enabled:SetChecked(profile.ResInfoEnabled)
	options.CastingColor:SetValue(HexToRGB(profile.ResInfoCastingColor))
	options.PendingColor:SetValue(HexToRGB(profile.ResInfoPendingColor))
	options.SelfResColor:SetValue(HexToRGB(profile.ResInfoSelfResColor))
end

function Addon:SaveOptions()
	local profile = PerfectRaid.db.profile

	profile.ResInfoEnabled = not not options.Enabled:GetChecked()
	profile.ResInfoCastingColor = RGBToHex(options.CastingColor:GetValue())
	profile.ResInfoPendingColor = RGBToHex(options.PendingColor:GetValue())
	profile.ResInfoSelfResColor = RGBToHex(options.SelfResColor:GetValue())

	statusColor.CASTING = profile.ResInfoCastingColor
	statusColor.PENDING = profile.ResInfoPendingColor
	statusColor.SELFRES = profile.ResInfoSelfResColor

	self:DONGLE_PROFILE_CHANGED("SaveOptions", "PerfectRaid_ResInfo", "PerfectRaidDB")
end